//
//  NSObject(MTLSControlItemEnabler).h
//  TravelAdvisor
//
//  Created by narita on 07/10/15.
//  Copyright 2007 MindTools Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>

/*!

NSWindowControllerのサブクラスやApplicationDelegateのようなレスポンダチェーン中のクラスで
以下のように呼び出す。
- (BOOL)validateUserInterfaceItem:(id<NSValidatedUserInterfaceItem>)anItem
{
    BOOL theResult = YES;

 [self respondsAndPerformStateAction:(id)anItem];
 [self respondsAndPerformImageAction:(id)anItem];
 [self respondsAndPerformTitleAction:(id)anItem];
 
     if([self respondsToCanSelector:[anItem action]])
     {
         theResult = [self performCanSelector:[anItem action]];
     }
     else if( [[self superclass] instancesRespondToSelector:@selector(validateUserInterfaceItem:)] == YES)
     {
         theResult = [(id)super validateUserInterfaceItem:anItem];
     }
 
	return theResult;
}

 ユーティリティーメソッドとして
 - (BOOL) respondsAndPerformAnyUtility:(id)inItem;
を用意した。
 - (BOOL)validateUserInterfaceItem:(id<NSValidatedUserInterfaceItem>)anItem
 {
     BOOL theResult = YES;
     
     [self respondsAndPerformAnyUtility:(id)anItem];
     
     if([self respondsToCanSelector:[anItem action]])
     {
         theResult = [self performCanSelector:[anItem action]];
     }
     else if( [[self superclass] instancesRespondToSelector:@selector(validateUserInterfaceItem:)] == YES)
     {
         theResult = [(id)super validateUserInterfaceItem:anItem];
     }
     
     return theResult;
 }
 で、同じ事が出来る。
 
*/

@interface NSObject (MTLSControlItemEnabler)

/*!
    @method     performCanSelector
    @abstract   与えられたアクションが事項できるかを"can"+セレクター名の文字列のメソッドを呼び出す事で判断する
    @discussion ib上のbindingを使用しないでactionの有効無効を実行時に変更するために使用する。
				アクション"selectNext:"を使用可能か判断するメソッドを"canSelectNext"と想定して判断する。

*/
- (BOOL) performCanSelector:(SEL)aSelector;

/*!
    @method     respondsToCanSelector
    @abstract   与えられたaSelectorから"canSelector"を合成してそのセレクターが存在するか判断する
    @discussion canPerformSelectorと組み合わせて使う事で、各種クラスのvalidateUserInterfaceItemの実装をサポートする

*/
- (BOOL) respondsToCanSelector:(SEL)aSelector;


/*!
 @method     respondsAndPerformAnyUtility:
 @abstract   レスポンダーチェーンで使うユーティリティーメソッド。詳細をいじらない場合はこのメソッド一発でよい
 @discussion 多少手直しする場合は、performCanSelectorやrespondsToを細かく呼ぶ事。
 */
- (void) respondsAndPerformAnyUtility:(id)inItem;

@end

/* Protocol implemented by validated objects */


/// State (NSMixedState, NSOffState, NSOnState)
#define MTLInterfaceItem_StateOf @"stateOf"
@protocol MTLStateUserInterfaceItem <NSValidatedUserInterfaceItem>
- (NSInteger)state;
- (void)setState:(NSInteger)value;
@end

@interface NSObject (MTLStateUserInterfaceItem)
- (BOOL) respondsToStateSelector:(SEL)aSelector;
- (NSInteger) performStateAction:(id <MTLStateUserInterfaceItem>)anItem;
- (void) respondsAndPerformStateAction:(id <MTLStateUserInterfaceItem>)anItem;
@end


/// Image
#define MTLInterfaceItem_ImageOf @"imageOf"
@protocol MTLImageUserInterfaceItem <NSValidatedUserInterfaceItem>
- (NSImage *)image;
- (void)setImage:(NSImage *)image;
@end

@interface NSObject (MTLImageUserInterfaceItem)
- (BOOL) respondsToImageSelector:(SEL)aSelector;
- (NSImage *) performImageAction:(id <MTLImageUserInterfaceItem>)anItem;
- (void) respondsAndPerformImageAction:(id <MTLImageUserInterfaceItem>)anItem;
@end


/// Title
#define MTLInterfaceItem_TitleOf @"titleOf"
@protocol MTLTitleUserInterfaceItem <NSValidatedUserInterfaceItem>
- (NSString*)title;
- (void)setTitle:(NSString*)aString;
@end

@interface NSObject (MTLTitleUserInterfaceItem)
- (BOOL) respondsToTitleSelector:(SEL)aSelector;
- (NSString*) performTitleAction:(id <MTLTitleUserInterfaceItem>)anItem;
- (void) respondsAndPerformTitleAction:(id <MTLTitleUserInterfaceItem>)anItem;
@end

// ToDo:NSMatrixでのラジオボタンやsegmentedControlのどれが選ばれているかのEnabler作成



@interface NSString (MTLStringExtensionMethods)
- (NSString*) stringByPrefixedString:(NSString*) aPrefix;
@end

