//
//  NSObject(MTLSControlItemEnabler).m
//  TravelAdvisor
//
//  Created by narita on 07/10/15.
//  Copyright 2007 MindTools Inc. All rights reserved.
//

#import "NSObject(MTLSControlItemEnabler).h"
#import <AppKit/NSUserInterfaceValidation.h>
#import <objc/objc.h>
#import <objc/runtime.h>
#import <objc/message.h>

// action文字列からcan文字列を生成する関数
// ToDo:遅い、最適化する事。mapで結果をキャッシュするとか
NSString* makeCanMethodFromActionMethod(NSString* inActionMethodString)
{    
    NSString*		theResultString;
    theResultString = [inActionMethodString stringByPrefixedString:@"can"];
    theResultString = [theResultString substringWithRange:NSMakeRange(0, [theResultString length] -1)];
    
	return theResultString;
}

// 文字列にプレフィックスを追加して返す
@implementation NSString (MTLStringExtensionMethods)

- (NSString*) stringByPrefixedString:(NSString*) aPrefix
{
    NSString*		theFirstString;
    NSString*		theOtherString;

	//	1文字目を取得し大文字にする
    theFirstString = [[self substringToIndex: 1] uppercaseString];
    
	//	2文字目以降
    theOtherString = [self substringWithRange:NSMakeRange(1, [self length] -1)];
    
	return [ aPrefix stringByAppendingString:[theFirstString stringByAppendingString:theOtherString]];
}

@end





@implementation NSObject (MTLSControlItemEnabler)

// canXXXメソッドの実行
- (BOOL) performCanSelector:(SEL)aSelector
{
    SEL		theCanSelector;
	BOOL	(*canDoingFunction)(id, SEL);
	
    theCanSelector = NSSelectorFromString(makeCanMethodFromActionMethod(NSStringFromSelector(aSelector)));

	canDoingFunction = ( BOOL (*)(id, SEL))[ self methodForSelector: theCanSelector];
	
	return (*canDoingFunction)(self, theCanSelector);
}

// aSelectorから"canSelecter"を生成しそのメソッドを実装しているか確認する
- (BOOL) respondsToCanSelector:(SEL)aSelector
{
    SEL			theCanSelector;

    theCanSelector = NSSelectorFromString(makeCanMethodFromActionMethod(NSStringFromSelector(aSelector)));
	
    if([ self respondsToSelector: theCanSelector ])
	{
		return YES;
	}

	return NO;
}

- (void) respondsAndPerformAnyUtility:(id)inItem
{
//    BOOL theResult = YES;
    
    [self respondsAndPerformStateAction:(id)inItem];
    [self respondsAndPerformImageAction:(id)inItem];
    [self respondsAndPerformTitleAction:(id)inItem];
    
//    if([self respondsToCanSelector:[inItem action]])
//    {
//        theResult = [self performCanSelector:[inItem action]];
//    }
//    else if( [[self superclass] instancesRespondToSelector:@selector(validateUserInterfaceItem:)] == YES)
//    {
//         theResult = [super validateUserInterfaceItem:inItem];
//    }
//    
//	return theResult;
}

@end





/// State (NSMixedState, NSOffState, NSOnState)
@implementation NSObject (MTLStateUserInterfaceItem)

// sateメソッドの存在確認
- (BOOL) respondsToStateSelector:(SEL)aSelector
{
    NSString* thePrefixedString;
    SEL       theSelector;
    
    thePrefixedString = [NSStringFromSelector(aSelector) stringByPrefixedString:MTLInterfaceItem_StateOf];
    
    theSelector = NSSelectorFromString(thePrefixedString);
	
    if( [self respondsToSelector: theSelector] )
	{
		return YES;
	}
    
	return NO;
}
// stateメソッドの実行
- (NSInteger) performStateAction:(id <MTLStateUserInterfaceItem>)anItem
{
    SEL		  theSelector;
	NSInteger (*doingFunction)(id, SEL, id);

    theSelector = NSSelectorFromString([NSStringFromSelector([anItem action]) stringByPrefixedString:MTLInterfaceItem_StateOf]);
	doingFunction = ( NSInteger (*)(id, SEL, id))[ self methodForSelector: theSelector];

    return (*doingFunction)(self, theSelector, anItem);
}

// stateメソッドの存在確認と実行
- (void) respondsAndPerformStateAction:(id <MTLStateUserInterfaceItem>)anItem
{
    if([self respondsToStateSelector:[anItem action]])
    {
        [anItem setState:[self performStateAction:anItem]];
    }
}

@end

/// Image
@implementation NSObject (MTLImageUserInterfaceItem)
- (BOOL) respondsToImageSelector:(SEL)aSelector
{
    NSString* thePrefixedString;
    SEL       theSelector;
    
    thePrefixedString = [NSStringFromSelector(aSelector) stringByPrefixedString:MTLInterfaceItem_ImageOf];
    
    theSelector = NSSelectorFromString(thePrefixedString);
	
    if( [self respondsToSelector: theSelector] )
	{
		return YES;
	}
    
	return NO;
}

- (NSImage *) performImageAction:(id <MTLImageUserInterfaceItem>)anItem
{
    SEL		  theSelector;
	NSImage* (*doingFunction)(id, SEL, id);
    
    theSelector = NSSelectorFromString([NSStringFromSelector([anItem action]) stringByPrefixedString:MTLInterfaceItem_ImageOf]);
	doingFunction = ( NSImage* (*)(id, SEL, id))[ self methodForSelector: theSelector];
    
    return (*doingFunction)(self, theSelector, anItem);
}

- (void) respondsAndPerformImageAction:(id <MTLImageUserInterfaceItem>)anItem
{
    if([self respondsToImageSelector:[anItem action]])
    {
        [anItem setImage:[self performImageAction:anItem]];
    }
}
@end

/// Title
@implementation NSObject (MTLTitleUserInterfaceItem)

- (BOOL) respondsToTitleSelector:(SEL)aSelector
{
    NSString* thePrefixedString;
    SEL       theSelector;
    
    thePrefixedString = [NSStringFromSelector(aSelector) stringByPrefixedString:MTLInterfaceItem_TitleOf];
    
    theSelector = NSSelectorFromString(thePrefixedString);
	
    if( [self respondsToSelector: theSelector] )
	{
		return YES;
	}
    
	return NO;
}

- (NSString*) performTitleAction:(id <MTLTitleUserInterfaceItem>)anItem
{
    SEL		  theSelector;
	NSString* (*doingFunction)(id, SEL, id);
    
    theSelector = NSSelectorFromString([NSStringFromSelector([anItem action]) stringByPrefixedString:MTLInterfaceItem_TitleOf]);
	doingFunction = ( NSString* (*)(id, SEL, id))[ self methodForSelector: theSelector];
    
    return (*doingFunction)(self, theSelector, anItem);
}

- (void) respondsAndPerformTitleAction:(id <MTLTitleUserInterfaceItem>)anItem
{
    if([self respondsToTitleSelector:[anItem action]])
    {
        [anItem setTitle:[self performTitleAction:anItem]];
    }
}

@end

